--
-- SplitNote.lua
-- ノートイベントを分割。
--
--
-- Copyright (C) 2012 ちょむＰ/VOCALOMAKETS
--

--
-- プラグインマニフェスト関数.
--
function manifest()
    myManifest = {
        name          = "SplitNote",
        comment       = "ノートイベントを分割。",
        author        = "ちょむＰ/VOCALOMAKETS",
        pluginID      = "{F1EAA432-3B4E-42bd-BBE1-6BAE1679F49C}",
        pluginVersion = "1.0.1.2",
        apiVersion    = "3.0.0.1"
    }
    return myManifest
end


--
-- スクリプトのエントリポイント関数.
--
function main(processParam, envParam)
	-- 実行時に渡されたパラメータを取得する.
	local beginPosTick = processParam.beginPosTick	-- 選択範囲の始点時刻（ローカルTick）.
	local endPosTick   = processParam.endPosTick	-- 選択範囲の終点時刻（ローカルTick）.
	local songPosTick  = processParam.songPosTick	-- カレントソングポジション時刻（ローカルTick）.

	-- 終了コード
	local endStatus = 0

	-- ノートリスト先頭に位置づける
	VSSeekToBeginNote()
	
	-- 先頭ノート取得
	local retCode, note = VSGetNextNoteEx()
	while (retCode == 1 ) do
		
		-- カーソル位置が選択範囲外なら終了
		if ( note.posTick >= endPosTick ) then 
			endStatus = 0
			break
 		end
		
		-- カーソル位置が選択範囲内、かつノートが[-]以外なら実行
		if ( note.posTick >= beginPosTick and note.phonemes ~= "-" ) then
			
			-- 16分音符(120Tick)より長い場合
			if (note.durTick > 120) then
				
				-- 取得したノートをとっておく
				local newNote = {
					posTick			= note.posTick,
					durTick			= note.durTick,
					noteNum			= note.noteNum,
					velocity		= note.velocity,
					lyric			= note.lyric,
					phonemes		= note.phonemes,
					phLock			= note.phLock,
					bendDepth		= note.bendDepth,
					bendLength		= note.bendLength,
					risePort		= note.risePort,
					fallPort		= note.fallPort,
					decay			= note.decay,
					accent			= note.accent,
					opening			= note.opening,
					vibratoLength	= note.vibratoLength,
					vibratoType		= note.vibratoType
				}
				
				-- ビブラート長（％）から実際のビブラートTick数を計算
				local vibTickAll = note.durTick * (note.vibratoLength / 100)
				
				-- 後続のノートからはみ出す分だけこのノートのビブラート長とする
				local vibTick = vibTickAll - (newNote.durTick - 120)
				if ( vibTick > 0 ) then
					-- ％に変換（小数点切り捨て）
					note.vibratoLength = math.floor( (vibTick / 120) * 100 )
				else
					-- かからないならビブラートしない
					note.vibratoType = 0
					note.vibratoLength = 0
				end
				
				-- ノートを16分音符に
				note.durTick = 120
				
				-- ノートを更新
				retCode = VSUpdateNoteEx(note)
				if (retCode ~= 1 ) then
					-- エラー
					endStatus = 1
					break
				end
				
				-- 新規ノートの用意
				
				-- 既存ノートの16分先
				newNote.posTick = newNote.posTick + 120
				
				-- 既存ノートの16分短い長さ
				newNote.durTick = newNote.durTick - 120
				
				-- ベロシティは64
				newNote.velocity = 64
				
				-- 歌詞は「-」
				newNote.lyric = "-"
				newNote.phonemes = "-"
				
				-- ノート長が短くなったのでビブラート長を再計算する
				if ( vibTickAll > 0 ) then
					newNote.vibratoLength = math.floor( (vibTickAll / newNote.durTick) * 100 )
					if ( newNote.vibratoLength > 100) then
						newNote.vibratoLength = 100
					end
				else
					-- かからないならビブラートしない
					newNote.vibratoType = 0
					newNote.vibratoLength = 0
				end
				
				-- 他は変更なし
				
				-- ノート追加
				retCode = VSInsertNoteEx(newNote)
				if (retCode ~= 1 ) then
					-- エラー
					endStatus = 1
					break
				end
			end
		end
		
		-- 次のノートを取得する.
		retCode, note = VSGetNextNoteEx()
	end
	
	return endStatus
end
